"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../constants");
const helpers = require("../../common/helpers");
const path = require("path");
const buildInfoFileName = ".nsbuildinfo";
class DeviceInstallAppService {
    constructor($analyticsService, $buildArtefactsService, $devicePathProvider, $fs, $logger, $mobileHelper, $buildInfoFileService, $projectDataService, $platformsDataService) {
        this.$analyticsService = $analyticsService;
        this.$buildArtefactsService = $buildArtefactsService;
        this.$devicePathProvider = $devicePathProvider;
        this.$fs = $fs;
        this.$logger = $logger;
        this.$mobileHelper = $mobileHelper;
        this.$buildInfoFileService = $buildInfoFileService;
        this.$projectDataService = $projectDataService;
        this.$platformsDataService = $platformsDataService;
    }
    installOnDevice(device, buildData, packageFile) {
        return __awaiter(this, void 0, void 0, function* () {
            this.$logger.out(`Installing on device ${device.deviceInfo.identifier}...`);
            const projectData = this.$projectDataService.getProjectData(buildData.projectDir);
            const platformData = this.$platformsDataService.getPlatformData(device.deviceInfo.platform, projectData);
            yield this.$analyticsService.trackEventActionInGoogleAnalytics({
                action: "Deploy",
                device,
                projectDir: projectData.projectDir
            });
            if (!packageFile) {
                packageFile = yield this.$buildArtefactsService.getLatestApplicationPackagePath(platformData, buildData);
            }
            yield platformData.platformProjectService.cleanDeviceTempFolder(device.deviceInfo.identifier, projectData);
            const platform = device.deviceInfo.platform.toLowerCase();
            yield device.applicationManager.reinstallApplication(projectData.projectIdentifiers[platform], packageFile);
            const outputFilePath = buildData.outputPath;
            yield this.updateHashesOnDevice({
                device,
                appIdentifier: projectData.projectIdentifiers[platform],
                outputFilePath,
                platformData
            });
            if (!buildData.release) {
                const deviceFilePath = yield this.getDeviceBuildInfoFilePath(device, projectData);
                const options = buildData;
                options.buildForDevice = !device.isEmulator;
                const buildInfoFilePath = outputFilePath || platformData.getBuildOutputPath(buildData);
                const appIdentifier = projectData.projectIdentifiers[platform];
                yield device.fileSystem.putFile(path.join(buildInfoFilePath, buildInfoFileName), deviceFilePath, appIdentifier);
            }
            this.$logger.out(`Successfully installed on device with identifier '${device.deviceInfo.identifier}'.`);
        });
    }
    installOnDeviceIfNeeded(device, buildData, packageFile) {
        return __awaiter(this, void 0, void 0, function* () {
            const shouldInstall = yield this.shouldInstall(device, buildData);
            if (shouldInstall) {
                yield this.installOnDevice(device, buildData, packageFile);
            }
        });
    }
    getDeviceBuildInfoFilePath(device, projectData) {
        return __awaiter(this, void 0, void 0, function* () {
            const platform = device.deviceInfo.platform.toLowerCase();
            const deviceRootPath = yield this.$devicePathProvider.getDeviceProjectRootPath(device, {
                appIdentifier: projectData.projectIdentifiers[platform],
                getDirname: true
            });
            return helpers.fromWindowsRelativePathToUnix(path.join(deviceRootPath, buildInfoFileName));
        });
    }
    shouldInstall(device, buildData) {
        return __awaiter(this, void 0, void 0, function* () {
            const projectData = this.$projectDataService.getProjectData(buildData.projectDir);
            const platformData = this.$platformsDataService.getPlatformData(device.deviceInfo.platform, projectData);
            const platform = device.deviceInfo.platform;
            if (!(yield device.applicationManager.isApplicationInstalled(projectData.projectIdentifiers[platform.toLowerCase()]))) {
                return true;
            }
            const deviceBuildInfo = yield this.getDeviceBuildInfo(device, projectData);
            const localBuildInfo = this.$buildInfoFileService.getBuildInfoFromFile(platformData, Object.assign({}, buildData, { buildForDevice: !device.isEmulator }));
            return !localBuildInfo || !deviceBuildInfo || deviceBuildInfo.buildTime !== localBuildInfo.buildTime;
        });
    }
    updateHashesOnDevice(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const { device, appIdentifier, platformData, outputFilePath } = data;
            if (!this.$mobileHelper.isAndroidPlatform(platformData.normalizedPlatformName)) {
                return;
            }
            let hashes = {};
            const hashesFilePath = path.join(outputFilePath || platformData.getBuildOutputPath(null), constants_1.HASHES_FILE_NAME);
            if (this.$fs.exists(hashesFilePath)) {
                hashes = this.$fs.readJson(hashesFilePath);
            }
            yield device.fileSystem.updateHashesOnDevice(hashes, appIdentifier);
        });
    }
    getDeviceBuildInfo(device, projectData) {
        return __awaiter(this, void 0, void 0, function* () {
            const deviceFilePath = yield this.getDeviceBuildInfoFilePath(device, projectData);
            try {
                const deviceFileContent = yield this.$mobileHelper.getDeviceFileContent(device, deviceFilePath, projectData);
                return JSON.parse(deviceFileContent);
            }
            catch (e) {
                return null;
            }
        });
    }
}
exports.DeviceInstallAppService = DeviceInstallAppService;
$injector.register("deviceInstallAppService", DeviceInstallAppService);
